@@ -1,6 +1,6 @@
 /* ecc-point.c
 
-   Copyright (C) 2013 Niels Möller
+   Copyright (C) 2013, 2014 Niels Möller
 
    This file is part of GNU Nettle.
 
@@ -42,13 +42,13 @@ void
 ecc_point_init (struct ecc_point *p, const struct ecc_curve *ecc)
 {
   p->ecc = ecc;
-  p->p = gmp_alloc_limbs (2*ecc->size);
+  p->p = gmp_alloc_limbs (2*ecc->p.size);
 }
 
 void
 ecc_point_clear (struct ecc_point *p)
 {
-  gmp_free_limbs (p->p, 2*p->ecc->size);
+  gmp_free_limbs (p->p, 2*p->ecc->p.size);
 }
 
 int
@@ -59,23 +59,42 @@ ecc_point_set (struct ecc_point *p, const mpz_t x, const mpz_t y)
   mpz_t t;
   int res;
 
-  size = p->ecc->size;
+  size = p->ecc->p.size;
   
-  if (mpz_sgn (x) < 0 || mpz_limbs_cmp (x, p->ecc->p, size) >= 0
-      || mpz_sgn (y) < 0 || mpz_limbs_cmp (y, p->ecc->p, size) >= 0)
+  if (mpz_sgn (x) < 0 || mpz_limbs_cmp (x, p->ecc->p.m, size) >= 0
+      || mpz_sgn (y) < 0 || mpz_limbs_cmp (y, p->ecc->p.m, size) >= 0)
     return 0;
 
   mpz_init (lhs);
   mpz_init (rhs);
 
-  /* Check that y^2 = x^3 - 3*x + b (mod p) */
   mpz_mul (lhs, y, y);
-  mpz_mul (rhs, x, x);
-  mpz_sub_ui (rhs, rhs, 3);
-  mpz_mul (rhs, rhs, x);
-  mpz_add (rhs, rhs, mpz_roinit_n (t, p->ecc->b, size));
+  
+  if (p->ecc->p.bit_size == 255)
+    {
+      /* ed25519 special case. FIXME: Do in some cleaner way? */
+      mpz_t x2;
+      mpz_init (x2);
+      mpz_mul (x2, x, x);
+      mpz_mul (rhs, x2, lhs);
+      /* Check that -x^2 + y^2 = 1 - (121665/121666) x^2 y^2
+	 or 121666 (1 + x^2 - y^2) = 121665 x^2 y^2 */
+      mpz_sub (lhs, x2, lhs);
+      mpz_add_ui (lhs, lhs, 1);
+      mpz_mul_ui (lhs, lhs, 121666);
+      mpz_mul_ui (rhs, rhs, 121665);
+      mpz_clear (x2);
+    }
+  else
+    {
+      /* Check that y^2 = x^3 - 3*x + b (mod p) */
+      mpz_mul (rhs, x, x);
+      mpz_sub_ui (rhs, rhs, 3);
+      mpz_mul (rhs, rhs, x);
+      mpz_add (rhs, rhs, mpz_roinit_n (t, p->ecc->b, size));
+    }
 
-  res = mpz_congruent_p (lhs, rhs, mpz_roinit_n (t, p->ecc->p, size));
+  res = mpz_congruent_p (lhs, rhs, mpz_roinit_n (t, p->ecc->p.m, size));
 
   mpz_clear (lhs);
   mpz_clear (rhs);
@@ -92,7 +111,7 @@ ecc_point_set (struct ecc_point *p, const mpz_t x, const mpz_t y)
 void
 ecc_point_get (const struct ecc_point *p, mpz_t x, mpz_t y)
 {
-  mp_size_t size = p->ecc->size;
+  mp_size_t size = p->ecc->p.size;
   if (x)
     mpz_set_n (x, p->p, size);
   if (y)
